"use strict";
exports.__esModule = true;
exports.StorageContext = void 0;
var CacheStrategy_1 = require("../base/CacheStrategy");
var iterators_1 = require("../iterators");
var LeafStorage_1 = require("./LeafStorage");
var storage_1 = require("../storage");
var RootLeafStrategy_1 = require("./RootLeafStrategy");
var strategy_1 = require("../strategy");
var constants_1 = require("../constants");
var utils_1 = require("../utils");
/**
 * Picks a `Noop` strategy if the `limit` is `Infinity`.
 * @param strategy
 * @param limit
 */
var pickNoopIfNoLimit = function (strategy, limit) { return (limit !== Infinity ? strategy : strategy_1.Noop); };
/**
 * Checks whether the provided limit is a valid number.
 * @param limit
 */
var limitIsValid = function (limit) {
    return (Number.isSafeInteger(limit) && limit > 0) || limit === Infinity;
};
/**
 * Ensures that the provided limit is a valid number.
 * @param name
 * @param value
 */
var ensureLimitIsValid = function (name, value) {
    if (!limitIsValid(value)) {
        throw new Error("`".concat(name, "` is invalid: ").concat(value));
    }
};
/**
 * Returns `true` if supplied class extends `CacheStrategy`.
 * @param strategy
 */
var extendsCacheStrategy = function (strategy) {
    return strategy &&
        strategy.prototype instanceof CacheStrategy_1.CacheStrategy;
};
/**
 * Root storage context.
 */
var StorageContext = /** @class */ (function () {
    function StorageContext(_a) {
        var _b = _a === void 0 ? utils_1.EMPTY_OBJECT : _a, _c = _b.strategy, strategy = _c === void 0 ? strategy_1.LRU : _c, _d = _b.totalLeavesLimit, totalLeavesLimit = _d === void 0 ? constants_1.DEFAULT_MAX_ENTRIES_COUNT : _d, _e = _b.totalStoragesLimit, totalStoragesLimit = _e === void 0 ? Infinity : _e, _f = _b.totalLeafStoragesLimit, totalLeafStoragesLimit = _f === void 0 ? Infinity : _f, _g = _b.leavesPerStorageLimit, leavesPerStorageLimit = _g === void 0 ? Infinity : _g, _h = _b.useWeakStorage, useWeakStorage = _h === void 0 ? false : _h, _j = _b.useObjectStorage, useObjectStorage = _j === void 0 ? false : _j, _k = _b.onCreateStorage, onCreateStorage = _k === void 0 ? utils_1.noop : _k, _l = _b.onRemoveStorage, onRemoveStorage = _l === void 0 ? utils_1.noop : _l, _m = _b.onCreateLeaf, onCreateLeaf = _m === void 0 ? utils_1.noop : _m, _o = _b.onRemoveLeaf, onRemoveLeaf = _o === void 0 ? utils_1.noop : _o;
        var leafStrategyClass, storageStrategyClass;
        if (extendsCacheStrategy(strategy)) {
            leafStrategyClass = strategy;
            storageStrategyClass = strategy;
        }
        else {
            var config = strategy;
            leafStrategyClass = config.leafStrategyClass;
            storageStrategyClass =
                config.storageStrategyClass;
        }
        if (!extendsCacheStrategy(leafStrategyClass)) {
            throw new Error("Invalid `leafStrategyClass`");
        }
        if (!extendsCacheStrategy(storageStrategyClass)) {
            throw new Error("Invalid `storageStrategyClass`");
        }
        ensureLimitIsValid("totalLeavesLimit", totalLeavesLimit);
        ensureLimitIsValid("totalStoragesLimit", totalStoragesLimit);
        ensureLimitIsValid("totalLeafStoragesLimit", totalLeafStoragesLimit);
        ensureLimitIsValid("leavesPerStorageLimit", leavesPerStorageLimit);
        storageStrategyClass = (0, CacheStrategy_1.withDestroyable)(pickNoopIfNoLimit(storageStrategyClass, totalStoragesLimit));
        leafStrategyClass =
            totalLeavesLimit === totalLeafStoragesLimit &&
                totalLeavesLimit === leavesPerStorageLimit
                ? pickNoopIfNoLimit(leafStrategyClass, totalLeavesLimit)
                : leafStrategyClass;
        this.rootStorageStrategy = new storageStrategyClass(totalStoragesLimit);
        this.rootLeafStrategy = new RootLeafStrategy_1.RootLeafStrategy(totalLeavesLimit, new ((0, CacheStrategy_1.withDestroyable)(leafStrategyClass))(totalLeafStoragesLimit));
        this.storageClass =
            useWeakStorage || useObjectStorage ? storage_1.UnifiedStorage : storage_1.SafeMapStorage;
        this.leafStrategyClass = leafStrategyClass;
        this.leavesPerStorageLimit = leavesPerStorageLimit;
        this.params = {
            useObjectStorage: useObjectStorage,
            useWeakStorage: useWeakStorage,
            onCreateStorage: onCreateStorage,
            onRemoveStorage: onRemoveStorage,
            onCreateLeaf: onCreateLeaf,
            onRemoveLeaf: onRemoveLeaf
        };
    }
    /**
     * Instantiates new storage with supplied parents
     * @param root
     */
    StorageContext.prototype.createStorage = function (parents) {
        if (parents === void 0) { parents = iterators_1.EMPTY_ITERABLE; }
        return new this.storageClass(this.params, parents);
    };
    /**
     * Instantiates new leaf storage with supplied parents
     * @param root
     */
    StorageContext.prototype.createLeafStorage = function (root) {
        if (root === void 0) { root = iterators_1.EMPTY_ITERABLE; }
        return new LeafStorage_1.LeafStorage(this.createStorage(), new this.leafStrategyClass(this.leavesPerStorageLimit), this.params, root);
    };
    return StorageContext;
}());
exports.StorageContext = StorageContext;
//# sourceMappingURL=StorageContext.js.map